/* MERG Content-to-template merger
   Command line:
      MERG [switches] template content1 content2 ...

   Valid switches:
      h        allow/Suppress HTML tags in content
      n        Keep/discard name
      l        long file name Acceptable/truncate
      e ERROR  Output error messages to ERROR
               instead of STD_ERR
      q        Quiet mode
      i GEN    Include file GEN for all content
               files
      s LIST   Suppress the sections defined in
               LIST

*/

#ifndef _MERG_H		/* Not included, merg.h */
#define _MERG_H		/* Okay, now I've included it */

#define _MERG_VER	"1.1"
#define _MERG_BUILD	"20011018/Beta"
#define _MERG_VMAJOR 1L
#define _MERG_VMINOR 1L

/*#define _MERG_DUPLICATESTREAM*/		/* Just to make the DuplicateStream version */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "wrapmalloc.h"

/* ****************************** */
/* #defines */

/* Token types */
#define TKN_UNKNOWN		0
#define TKN_START		1
#define TKN_TEXT		2
#define TKN_NEWLINE		3

#define TKN_MARKUP		16
#define TKN_HTML		TKN_MARKUP + 1
#define TKN_COMMENT		TKN_MARKUP + 2

#define TKN_T_COMMAND	TKN_MARKUP + 8
#define TKN_T_INSERT	TKN_COMMAND + 1
#define TKN_T_SCRIPT	TKN_COMMAND + 2
#define TKN_T_RESERVED	TKN_COMMAND + 3

#define TKN_ENDSTREAM	255

/* Flag constants for the flag int */
#define B_HTML		1
#define B_QUIET	2
#define B_LFN		4
#define B_DEBUG	8
#define B_SIMUL	16
#define B_FULLDEBUG	B_DEBUG+B_SIMUL
#define B_NAME		32
#define B_SUPPRESS	64
#define B_ERROR		128

/* Data ID values for extensions */
#define XID_EMPTY	0
#define XID_CHAR	1
#define XID_INT	2
#define XID_LONG	3
#define XID_BUFFER	4
#define XID_COMMANDSET	5
#define XID_EXTENSION	6
#define XID_TOKEN	7
#define XID_TSTREAM	8
#define XID_UNDEFINED	255

/* Commands */
#define CMD_NONE	0
#define CMD_ADD	1
#define CMD_BUILD	2
#define CMD_ERR	3
#define CMD_UNKNOWN	255

/* Misc. constants */
#define BUFFSIZE	4096		/* Size of buffers used */

/* ****************************** */
/* structs/typedefs */

typedef struct _tagNAMESPACE {
	char * name;
	int offset;
} NAMESPACE;

typedef struct _tagEXTENSION {
	NAMESPACE * names;
	int nDataID;
	long nDataDesc;
	void * data;
} EXTENSION;

typedef struct _tagCOMMANDSET {
	int nCommand;
	char * data;
} COMMANDSET;

typedef struct _tagTOKEN {
	char * szContent;
	char * strTokenName;
	int nTokenID;
	EXTENSION * extension;
} TOKEN;

typedef struct _tagTKNSTREAM {
	char * streamname;
	char * strings;
	int count, offset, boffset;
	int namecount;
	NAMESPACE * names;
	TOKEN * start;
} TOKENSTREAM;

extern int tknDirect;

/* ****************************** */
/* prototypes */

/* merg.c - UI related */
void ErrOut (char *);

/* help.c */
void help(void);
void header(void);
void version(void);

/* mparse.c */
TOKENSTREAM * ParseTemplate(char *, int);
int ParseContent(TOKENSTREAM *, char *, int, FILE *);

/* token.c */
char * TokenContent (TOKEN *, char *);
char * TokenName (TOKEN *, char *);
int TokenType (TOKEN *, int);
int FreeToken (TOKEN *);
TOKEN * CreateToken (TOKEN *);

/* mergapi.c */


/* tstream.c */
TOKENSTREAM * CreateStream(void);

#if defined(_MERG_DUPLICATESTREAM)
TOKENSTREAM * DuplicateStream(TOKENSTREAM *);
#endif
int FreeStream (TOKENSTREAM *);

inline int StreamOffsetToEnd(TOKENSTREAM *);
inline int StreamOffsetToStart(TOKENSTREAM *);
int StreamOffset(TOKENSTREAM *, int);
int StreamOffsetAbs(TOKENSTREAM *, int);
inline int StreamCount (TOKENSTREAM *);

int AddToken (TOKENSTREAM *, TOKEN *);
int DeleteToken (TOKENSTREAM *, TOKEN *);
int DeleteTokenO (TOKENSTREAM *, int);

int AttachName(TOKENSTREAM *, int);
int SetStreamName(TOKENSTREAM *, char *);

inline int OffsetToken (TOKENSTREAM *);
TOKEN * GetToken (TOKENSTREAM *, int);
int SearchToken (TOKENSTREAM *, TOKEN *);
TOKEN * MatchToken (TOKENSTREAM *, char *);
TOKEN * FindTokenType (TOKENSTREAM *, int);
void WalkStream (TOKENSTREAM *, FILE *);

inline NAMESPACE * GetNamespace (TOKENSTREAM *);


#endif		/* merg.h already included */
